"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const openapi_typescript_1 = __importDefault(require("openapi-typescript"));
const node_core_library_1 = require("@rushstack/node-core-library");
const PLUGIN_NAME = 'openapi-typescript-plugin';
class OpenApiTypescriptPlugin {
    apply(heftSession, heftConfiguration, options) {
        heftSession.hooks.run.tapPromise(PLUGIN_NAME, async () => {
            await this._runOpenApiTypescriptAsync(options, heftSession, heftConfiguration);
        });
    }
    async _runOpenApiTypescriptAsync(options, { logger }, { buildFolderPath }) {
        const resolvedEntries = [];
        for (const entry of options.entries) {
            resolvedEntries.push(Object.assign(Object.assign({}, entry), { resolvedSourcePath: path.resolve(buildFolderPath, entry.sourcePath), resolvedOutputPath: path.resolve(buildFolderPath, entry.outputPath) }));
        }
        await node_core_library_1.Async.forEachAsync(resolvedEntries, async (entry) => {
            await this._generateOpenApiTypescript(entry, logger);
        }, { concurrency: 5 });
    }
    async _generateOpenApiTypescript(entry, logger) {
        let fileContents;
        try {
            fileContents = await node_core_library_1.FileSystem.readFileAsync(entry.resolvedSourcePath);
        }
        catch (error) {
            if (node_core_library_1.FileSystem.isNotExistError(error)) {
                logger.emitError(new Error(`OpenAPI file not found: ${entry.sourcePath}`));
                await node_core_library_1.FileSystem.deleteFileAsync(entry.resolvedOutputPath);
                return;
            }
            else {
                throw error;
            }
        }
        let parsedApiFile;
        try {
            if (entry.resolvedSourcePath.endsWith('.yaml')) {
                const { default: jsYaml } = await Promise.resolve().then(() => __importStar(require('js-yaml')));
                parsedApiFile = jsYaml.safeLoad(fileContents);
            }
            else if (entry.resolvedSourcePath.endsWith('.json')) {
                parsedApiFile = node_core_library_1.JsonFile.parseString(fileContents);
            }
            else {
                logger.emitError(new Error(`Unsupported OpenAPI file format: ${entry.sourcePath}`));
                return;
            }
        }
        catch (error) {
            logger.emitError(new Error(`Failed to parse OpenAPI file ${entry.sourcePath}: ${error}`));
            return;
        }
        let output;
        try {
            output = await (0, openapi_typescript_1.default)(parsedApiFile);
        }
        catch (error) {
            logger.emitError(new Error(`Error generating typescript from OpenAPI file ${entry.sourcePath}: ${error}`));
            return;
        }
        try {
            await node_core_library_1.FileSystem.writeFileAsync(entry.resolvedOutputPath, output, {
                ensureFolderExists: true
            });
        }
        catch (error) {
            logger.emitError(new Error(`Error writing typescript file to ${entry.outputPath}: ${error}`));
        }
    }
}
exports.default = OpenApiTypescriptPlugin;
//# sourceMappingURL=OpenApiTypescriptPlugin.js.map